import 'package:flutter/material.dart';

class SplitTwo extends StatefulWidget {
  const SplitTwo({
    required this.childA,
    required this.childB,
    this.isVertical = false,
    this.dividerPosition = 0.5,
    this.dividerPositionMin = 0.2,
    this.dividerPositionMax = 0.8,
    this.dividerSize = 5,
    this.dividerColor,
    this.dividerColorActive,
    Key? key,
  }) : super(key: key);

  final Widget childA;
  final Widget childB;
  final bool isVertical;

  final double dividerPosition;
  final double dividerPositionMax;
  final double dividerPositionMin;
  final double dividerSize;
  final Color? dividerColor;
  final Color? dividerColorActive;

  @override
  _SplitState createState() => _SplitState();
}

class _SplitState extends State<SplitTwo> {
  late double _dividerPos; // 0-1
  Color? _dividerColor;

  Widget _buildHorizontal() {
    return LayoutBuilder(builder: (context, constraints) {
      final dividerX = constraints.maxWidth * _dividerPos - (widget.dividerSize / 2);
      final contentWidth = constraints.maxWidth - 1;
      final widthA = contentWidth * _dividerPos;
      final widthB = contentWidth - widthA;

      final divider = MouseRegion(
        cursor: SystemMouseCursors.resizeColumn,
        child: GestureDetector(
          behavior: HitTestBehavior.translucent,
          onHorizontalDragStart: (details) {
            setState(() {
              _dividerColor = widget.dividerColorActive ?? Theme.of(context).colorScheme.secondary;
            });
          },
          onHorizontalDragEnd: (details) {
            setState(() {
              _dividerColor = null;
            });
          },
          onHorizontalDragUpdate: (details) {
            setState(() {
              _dividerPos += details.delta.dx / contentWidth;
              if (_dividerPos > widget.dividerPositionMax) {
                _dividerPos = widget.dividerPositionMax;
              } else if (_dividerPos < widget.dividerPositionMin) {
                _dividerPos = widget.dividerPositionMin;
              }
            });
          },
          child: Container(
            color: _dividerColor,
          ),
        ),
      );

      return Stack(
        children: [
          Row(
            children: <Widget>[
              SizedBox(
                width: widthA,
                child: widget.childA,
              ),
              Container(
                width: 1,
                color: widget.dividerColor ?? Theme.of(context).backgroundColor,
              ),
              SizedBox(
                width: widthB,
                child: widget.childB,
              ),
            ],
          ),
          Positioned(
            width: widget.dividerSize,
            height: constraints.maxHeight,
            left: dividerX,
            child: divider,
          ),
        ],
      );
    });
  }

  Widget _buildVertical() {
    return LayoutBuilder(builder: (context, constraints) {
      final dividerY = constraints.maxHeight * _dividerPos - (widget.dividerSize / 2);
      final contentHeight = constraints.maxHeight - 1;
      final heightA = contentHeight * _dividerPos;
      final heightB = contentHeight - heightA;

      final divider = MouseRegion(
        cursor: SystemMouseCursors.resizeRow,
        child: GestureDetector(
          behavior: HitTestBehavior.translucent,
          onVerticalDragStart: (details) {
            setState(() {
              _dividerColor =
                  widget.dividerColorActive ?? Theme.of(context).colorScheme.secondaryVariant;
            });
          },
          onVerticalDragEnd: (details) {
            setState(() {
              _dividerColor = null;
            });
          },
          onVerticalDragUpdate: (details) {
            setState(() {
              _dividerPos += details.delta.dy / contentHeight;
              if (_dividerPos > widget.dividerPositionMax) {
                _dividerPos = widget.dividerPositionMax;
              } else if (_dividerPos < widget.dividerPositionMin) {
                _dividerPos = widget.dividerPositionMin;
              }
            });
          },
          child: Container(
            color: _dividerColor,
          ),
        ),
      );

      return Stack(
        children: [
          Column(
            children: <Widget>[
              SizedBox(
                height: heightA,
                child: widget.childA,
              ),
              Container(
                height: 1,
                color: widget.dividerColor ?? Theme.of(context).backgroundColor,
              ),
              SizedBox(
                height: heightB,
                child: widget.childB,
              ),
            ],
          ),
          Positioned(
            width: constraints.maxWidth,
            height: widget.dividerSize,
            top: dividerY,
            child: divider,
          ),
        ],
      );
    });
  }

  @override
  Widget build(BuildContext context) {
    return widget.isVertical ? _buildVertical() : _buildHorizontal();
  }

  @override
  void initState() {
    super.initState();
    _dividerPos = widget.dividerPosition;
  }
}
